import bpy

# Edit the file name here to change it to match the file you want to
# open.
file = open('INPUT_FILE.txt')
theLines = file.readlines()

#stolen from http://wiki.blender.org/index.php/Dev:2.5/Py/Scripts/Cookbook/Code_snippets/Materials_and_textures
#are we allowed to have this? :)
def setMaterial(ob, mat):
    me = ob.data
    me.materials.append(mat)

def makeMaterial(name, diffuse, specular, alpha):
    mat = bpy.data.materials.new(name)
    mat.diffuse_color = diffuse
    mat.diffuse_shader = 'LAMBERT' 
    mat.diffuse_intensity = 1.0 
    mat.specular_color = specular
    mat.specular_shader = 'COOKTORR'
    mat.specular_intensity = 0.5
    mat.alpha = alpha
    mat.ambient = 1
    return mat

# Replaces select_name() function that was removed in
# version 2.62.  The select_pattern() function does not quite
# do the same thing.
def selectNamed(name):
    bpy.ops.object.select_all(action='DESELECT')    
    myobject = bpy.data.objects[name]
    myobject.select = True
    bpy.context.scene.objects.active = myobject
    
# Faster version of the above that pulls the entry from the
# specified entry in a list
def selectObject(myObject):
    bpy.ops.object.select_all(action='DESELECT')
    myObject.select = True
    bpy.context.scene.objects.active = myObject

#n is the file line counter
n=0

frame_num = 1

# Create some materials
red = makeMaterial('Red', (1,0.5,0.5), (1,1,1), 1)
blue = makeMaterial('Blue', (0.5,0.5,1), (1,1,1), 1)
green = makeMaterial('Green', (0.5,1,0.5), (1,1,1), 1)
purple = makeMaterial('Purple', (1,0.5,1), (1,1,1), 1)
white = makeMaterial('White', (1,1,1),(1,1,1),1)

# Make a single sphere that will be duplicated to make the colored spheres
bpy.ops.mesh.primitive_uv_sphere_add(size = 0.0055, location=(1000.0,1000.0,1000.0))

# Make a sphere of each color to use as a basis for duplication.
# Put them far from the origin so they won't be seen.
# Put pointers to them into an array so that we can quickly find them
# again and we don't have to look them up by name.
sphereList = list()
selectNamed("Sphere")
bpy.ops.object.duplicate(linked=False)
setMaterial(bpy.context.object, red)
sphereList.append(bpy.context.scene.objects.active)

selectNamed("Sphere")
bpy.ops.object.duplicate(linked=False)
bpy.context.object.data.materials.append(blue)
sphereList.append(bpy.context.scene.objects.active)

selectNamed("Sphere")
bpy.ops.object.duplicate(linked=False)
setMaterial(bpy.context.object, green)
sphereList.append(bpy.context.scene.objects.active)

selectNamed("Sphere")
bpy.ops.object.duplicate(linked=False)
setMaterial(bpy.context.object, purple)
sphereList.append(bpy.context.scene.objects.active)

selectNamed("Sphere")
bpy.ops.object.duplicate(linked=False)
setMaterial(bpy.context.object, white)
sphereList.append(bpy.context.scene.objects.active)

#sphereNum is the sphere number counter
sphereNum = 1
i = 0
firstSet = True;

#for each line in the file, we're going to add a keyframe, and set the new sphere location
# Put pointers to them into an array so that we can quickly find them
# again and we don't have to look them up by name.
pointList = list()

# The first part of the file has the description of the structure
# we are using; it ends with a blank line.
# The first second of the file has a description of the mass colors.
# We put them into a list.
# The later sections describe the positions of masses.
parsingStructure = True;
parsingColor = True;
gotColorHeader = False;
colorLines = [];
for line in theLines:
    # Parse the structure lines.
    if (parsingStructure == True):
        # If we only have a blank line, then we've found the end.
        if not line.strip():
            parsingStructure = False;
        # Done with this line in any case.
        continue

    # Parse the color lines.
    if (parsingColor == True):
        if (gotColorHeader == False):
            #Skipping the header line.
            gotColorHeader = True;
        else:
            #Add this line to the color lines until we find an empty line
            #When we find the blank line, we're past the color section.
            try:
                val = int(line)
            except ValueError:
                val = 0
            if (val > 0):
                colorLines.append(line)
            else:
                parsingColor = False;
        #Parsed the line as a color line, so we're done with it.
        continue

    currentLine = line.split(" ")
    #if we're on a "time" line, then increase frame counter
    if len(currentLine) < 3:
        # Don't add a time step for blank lines.
        if len(currentLine) >= 2:
            # Set the frame time to the current frame_num and
            # then increment frame_num
            bpy.context.scene.frame_set(frame_num)
            print("Handling frame ", frame_num, ", time = ", currentLine[1])
            frame_num += 1
            
        #set the boolean flag for first set
        if (n != 0):
            firstSet = False;

        #increment the line counter
        n+=1
        
        # Reset the point-ID counter.
        i = 0
        
    else:
        # Make the names at the end of the alphabet and make their integer order
        # match their string order by padding to make sorting fast.
        myname = 'ZP'+str(i).zfill(5)

        #if we're on the first set then create the spheres.
        if (firstSet == True):
            bpy.ops.object.select_all(action='DESELECT')
            if(len(colorLines) >= sphereNum):
                if (colorLines[sphereNum-1] == "4\n"):
                    selectObject(sphereList[4])
                elif (colorLines[sphereNum-1] == "3\n"):
                    selectObject(sphereList[3])
                elif (colorLines[sphereNum-1] == "2\n"):
                    selectObject(sphereList[2])
                elif (colorLines[sphereNum-1] == "1\n"):
                    selectObject(sphereList[1])
                else: 
                    selectObject(sphereList[0])
            else:
                selectObject(sphereList[0])
            bpy.ops.object.duplicate(linked=True)
            bpy.context.active_object.name = myname
            pointList.append(bpy.context.scene.objects.active)
            sphereNum=sphereNum+1
    
        # Always set the location and the keyframe for the object.
        selectObject(pointList[i])
        bpy.context.active_object.location = (float(currentLine[0])*1e6, float(currentLine[1])*1e6, float(currentLine[2])*1e6)
        bpy.context.active_object.keyframe_insert(data_path='location')
#        bpy.ops.anim.keyframe_insert(type='Location', confirm_success=False)
        
        if (i % 100 == 0):
            print("  Handled line ", i)

        # Prepare to handle the next point.
        i=i+1

# However many time steps we found should define the movie length
bpy.data.scenes["Scene"].frame_end = bpy.data.scenes["Scene"].frame_current

#bpy.ops.screen.animation_play(reverse=False, sync=False)
bpy.ops.wm.save_mainfile(check_existing=False)
